
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.quartz.CronScheduleBuilder;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.quartz.impl.DirectSchedulerFactory;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.spi.JobStore;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

public class JobRunner {

	private List<String[]> rows = new ArrayList<String[]>();
    private List<Trigger> jobKeys = new ArrayList<Trigger>();
    private String path;
    Scheduler scheduler;
    DirectSchedulerFactory instance;
    JobStore jobStore;


	
	public JobRunner(String path) throws SchedulerException{
		this.path = path;
		FileWatcher fileWatcher = new FileWatcher(path, new Callback(this));
		fileWatcher.watch();  
	}
	
	//process the data
	/**
	 * @throws SchedulerException
	 */
	void run() throws SchedulerException {
		readCSV();
		
		
		try {
			 scheduler = new StdSchedulerFactory().getScheduler();
		   	scheduler.start();
			
			
			for (String[] row : this.rows) {
				System.out.println(row.length);
				//System.out.println(row[9]);
				JobDetail job = JobBuilder.newJob(ExcelJob.class).withIdentity("job-" + row[0], "group-" + row[0]).build();
				job.getJobDataMap().put("row", row);
				job.getJobDataMap().put("cronexpression", row);
				Trigger trigger = TriggerBuilder.newTrigger().withIdentity("cronTrigger-" + row[0], "group-" + row[0])
						.withSchedule(CronScheduleBuilder.cronSchedule(row[9])).build();
				
				scheduler.scheduleJob(job, trigger);
				jobKeys.add(trigger);
			}
			
		
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// Class to listen changes and callback action
	static class FileWatcher implements Runnable {

		private File file;
		private long lastModified;
		private Callback callback;

		public FileWatcher(String path, Callback callBack) {
			file = new File(path);
			lastModified = file.lastModified();
			this.callback = callBack;
		}

		public void watch() {
			Thread thread = new Thread(this);
			thread.start();
        }

		public void run() {
			while (true) {
				if (!(lastModified == file.lastModified())) {
					lastModified = file.lastModified();
					System.out.println("file modified");
					callback.onFileModify();
				}
			}

		}
	}

	 static class Callback {
		JobRunner myApp;

		private Callback(JobRunner app) {
			this.myApp = app;
		}

		public void onFileModify() {
			myApp.refresh();

		}

	}

	//read the CSV file data 
	public void readCSV() {
		BufferedReader br = null;
		String line = "";
		String cvsSplitBy = ",";

		try {

			br = new BufferedReader(new FileReader(path));
			while ((line = br.readLine()) != null) {

				String[] row = line.split(",");
				for (String string : row) {
					System.out.println(string);
					
				}
				
				System.out.println(row);
				rows.add(row);

			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

	}

	public void refresh() {
		System.out.println("refresh()");
		shutdown();
		jobKeys.clear();
		try {
			run();
		} catch (SchedulerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	

	public void shutdown() {
		System.out.println("shutdown()");
		try {
		for(Trigger sc:jobKeys) {
			scheduler.unscheduleJob(sc.getKey());
			
		}
		Thread.sleep(10000);
		run();
			
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
	

}